% Short script to calculate LCx,t values, with confidence intervals if a 
% sample from parameter space is available (with slice sampler).
% This script is for now only useful for the GUTS packages
% 
% Author : Tjalling Jager 
% Date:   Aug 2015
% Web support: http://www.debtox.info/byom.html

global names glo
if isempty(names) % if this script is run without fitting, 'names' is  not defined
    names  = fieldnames(par_out); % extract all field names of par (global)
end % this should not be needed anymore
if exist('type_conf','var') == 0 % if the variable type_conf does not exist ...
    type_conf = 1; % by default, assumes there is an MCMC sample
end

disp('')
disp('Calculate LCx values')
drawnow % clear the plotting buffer

Tend   = sort(Tend); % make sure that time increases
if Tend(1) == 0 % zero won't work as there is no effect there
    Tend(1) = []; % just remove the first entry
end
LCx    = zeros(length(Tend),1);
cmean  = mean(X0mat(1,:)); % mean exposure concentration in set as first guess for LC50
X0     = X0mat(2:end,1); % Note: I take the first column of X0mat for the 
% starting values of all state variable for the calculation of LCx!

S0     = call_deri([0 Tend(1)/2 Tend(1)],par_out,[0 X0']); % survival in control for Tend1
LCx(1) = fzero(@calc_lc50,cmean,[],Tend(1),par_out,Feff,X0mat(2:end,1),S0(end,glo.locS)); % find the first LCx,t

if length(Tend) > 1
    for i = 2:length(Tend) % run through remaining time points for the LC50
        S0     = call_deri([0 Tend(i)/2 Tend(i)],par_out,[0 X0']); % survival in control for Tend(i)
        LCx(i) = fzero(@calc_lc50,[0 1.05*LCx(i-1)],[],Tend(i),par_out,Feff,X0,S0(end,glo.locS)); % find the LCx,t
        % make use of the fact that LC50s always decrease in time!
    end
end

if LCx_plot == 1 % if required, make a plot!
    figure
    hold on
    plot(Tend,LCx,'k-','LineWidth',1)
    xlabel(xlab,'FontSize',12)
    ylabel(['LC',num2str(Feff*100),' (',leglab2,')'],'FontSize',12)
    xlim([0 Tend(end)])
    ylim([0 1.1*max(LCx)])
    set(gca,'LineWidth',1,'FontSize',12) % adapt axis formatting
    drawnow
end

switch type_conf
    case 1 % sample from posterior distribution, take 95% of MODEL CURVES
        if exist([mfilename_base,'_MC.mat']) ~= 2
            disp('There is no MCMC sample saved, so run calc_slice first!')
            disp('=================================================================================')
            for i = 1:length(Tend)
                fprintf('LC%1.0f (%1.0f days): %g \n',Feff*100,Tend(i),LCx(i))
            end
            disp('=================================================================================')
            return
        end
        load([mfilename_base,'_MC']) % load the random sample from the last MCMC run
        % this loads the random sample in rnd, and the parameter matrix par and the
        % selection matrix par_sel
        
    case 2 % LHS sample from joint conf. region, take min and max of curves
        if exist([mfilename_base,'_LR.mat']) ~= 2
            disp('There is no confidence set saved, so run calc_likregion first!')
            disp('=================================================================================')
            for i = 1:length(Tend)
                fprintf('LC%1.0f (%1.0f days): %g \n',Feff*100,Tend(i),LCx(i))
            end
            disp('=================================================================================')
            return
        end
        load([mfilename_base,'_LR']) % load the random sample from the last calc_likregion run
        % this loads the sets from the conf region in rnd, and the parameter matrix par and the
        % selection matrix par_sel
        rnd = acc; % use only the ACCEPTED sample to calculate the confidence bounds
        
    case 3 % ACCEPTED sample from posterior distribution (highest 95% posterior density), take min and max on curves
        if exist([mfilename_base,'_MC.mat']) ~= 2
            disp('There is no MCMC sample saved, so run calc_slice first!')
            disp('=================================================================================')
            for i = 1:length(Tend)
                fprintf('LC%1.0f (%1.0f days): %g \n',Feff*100,Tend(i),LCx(i))
            end
            disp('=================================================================================')
            return
        end
        load([mfilename_base,'_MC']) % load the random sample from the last MCMC run
        % this loads the random sample in rnd, and the parameter matrix par and the
        % selection matrix par_sel
        rnd = acc; % use only the ACCEPTED sample to calculate the confidence bounds
end

% this is for the progress displayer
progr = [0 10 20 30 40 50 60 70 80 90 100; 0 0 0 0 0 0 0 0 0 0 0];
progr_last = 0;
fprintf('Percentage of samples finished for CIs: ')

drawnow % empty plot buffer if there's something in it

n_samples = size(rnd,1); % number of MCMC samples
LC_coll   = zeros(n_samples,length(Tend)); % initialise vector to collect LCx values
pmat      = packunpack(1,par_out,0); % transform structure into a regular matrix
pfit      = pmat(pmat(:,2)==1,1); % parameter values that are to be estimated

for k = 1:n_samples, % run through all samples
    
    pmat(pmat(:,2)==1,1) = rnd(k,:); % replace values in pars with the k-th random sample from the MCMC
    
    % put parameters that need to be fitted on log scale back on normal
    % scale (as call_deri requires normal scale, in contrast to transfer.m)
    ind_logfit = (pmat(:,5)==0 & pmat(:,2)==1); % indext to pars on log scale that are also fitted!
    if sum(ind_logfit)>0
        pmat(ind_logfit,1) = 10.^(pmat(ind_logfit,1));
    end
    % Note: pmat is on normal scale here, but the sample in rnd contains
    % the value on a log scale, if a parameter is fitted on log scale.
    
    par_k = packunpack(2,0,pmat); % transform parameter matrix into a structure
    for i = 1:length(Tend)
        S0           = call_deri([0 Tend(i)/2 Tend(i)],par_k,[0 X0']); % survival in control for Tend(i)
        LC_coll(k,i) = fzero(@calc_lc50,LCx(i),[],Tend(i),par_k,Feff,X0,S0(end,glo.locS)); % find the LCx,t
        % make use of the fact that all LC50s are close to the best value
        % Note: I take the first column of X0mat for the starting
        % values of all state variable for the calculation of LCx!
    end
    
    % display progress in intervals given in progr
    if  100*(k/n_samples)> progr_last
        fprintf('%4.0f',progr_last)
        progr_last = progr(1,find(progr(1,:)==progr_last)+1);
        progr(2,progr(1,:)==progr_last) = 1;
    end
end
fprintf('\n') % end the line opened by the progress monitor

LClo = prctile(LC_coll,2.5,1);  % 2.5 percentile of LCx
LChi = prctile(LC_coll,97.5,1); % 97.5 percentile of LCx

disp('=================================================================================')
for i = 1:length(Tend)
    fprintf('LC%1.0f (%4g days): %g (%g-%g) \n',Feff*100,Tend(i),LCx(i),LClo(i),LChi(i))
end
disp('=================================================================================')

if LCx_plot == 1 % if required, make a plot!
    plot(Tend,LClo,'k:','LineWidth',1)
    plot(Tend,LChi,'k:','LineWidth',1)
    ylim([0 1.1*max(LChi)])
end

switch type_conf
    case 1
        title('Intervals: 95% of model predictions from sample of posterior')
        disp('Intervals: 95% of model predictions from sample of posterior')
    case 2
        title('Intervals: min-max of model predictions from 95% likelihood-based region')
        disp('Intervals: min-max of model predictions from 95% likelihood-based region')
    case 3
        title('Intervals: min-max of model predictions from 95% highest-posterior density region')
        disp('Intervals: min-max of model predictions from 95% highest-posterior density region')
end
disp(' ')

disp(['Time required: ' secs2hms(toc)])