% This script does some preliminary checks on the data set. These checks
% were earlier part of the files calc_and_plot and start_calc, but it is
% better to do them always (start_calc is only called when fitting).
% 
% Author : Tjalling Jager 
% Date:   June 2015
% Web support: http://www.debtox.info/byom.html

% ========================================================================
%                       PRELIMINARY CHECKS 
% ========================================================================

global n_D n_X names ttot ctot n_X2

n_s = size(X0mat,2); % number of scenarios
[n_D,n_X] = size(DATA);  % number of data sets per state and state variables
n_X = size(X0mat,1)-1; % take number of states from X0mat instead of data set
ndata = n_D * n_X;

if exist('DATAx') && ~isempty(DATAx)
    n_X2 = size(DATAx,2); % extra uni-variate dataset
else
    n_X2 = 0;
end

add_data=0; % check if empty data sets have been added
for i = 1:n_X % catch cases where data were forgotten ...
    if size(DATA,2) < i
        DATA{1,i} = 0;
        add_data = 1;
    end
end
for i = 1:ndata % run through state variables
    if isempty(DATA{i}) % catch cases where data were forgotten ...
        DATA{i} = 0;
        add_data = 1;
    end
end
if n_D > 1 && add_data == 1 % data have been added, so a reshape is needed
    DATA = reshape(DATA,n_D,n_X);
end
[n_D,n_X] = size(DATA);  % number of data sets per state and state variables
    
% if axis and legend labels are forgotten, just give default names
if ~exist('ylab') || isempty(ylab{1})
        ylab{1} = 'State var. 1';
    end
if n_X ~= size(ylab,2)
    for i = 2:n_X
        if length(ylab) < i
            ylab{i} = ['State var. ',num2str(i)];
        end
    end
end
if ~exist('xlab')
    xlab = 'time';
end
if ~exist('leglab1')
    leglab1 = 'Scen. '; 
end
if ~exist('leglab2')
    leglab2 = ''; 
end

if n_X ~= size(X0mat,1)-1 % this should not occur anymore
    error('The number of state variables in X0mat does not match the number of data sets entered. If you do not have data for a state, use DATA{x}=0.')
end
   
if ~exist('repls') % when using older script files, repls may not be defined
    repls = 1; % so just plot all replicates as defauls
end

if ~isfield(glo,'Tinit')
    glo.Tinit = 0; % if not defined, assume that t=0 is the start of the calculations
end
if t(1) ~= glo.Tinit % if the long time vector does not match the initial time ...
    t = linspace(glo.Tinit,max(t),length(t));
    % Create a new vector of same length that starts at glo.Tinit as we
    % have to start the model at the correct values for the initial states!
    % The time vector in the data set is checked in calc_start.
end    

if ~isfield(glo,'opt')
    glo.opt = [1 2]; % by default, use two simplex runs
else
    if glo.opt(1) == 3 
        if length(glo.opt) == 1 % swarm, but some info is missing
            glo.opt(2:3) = [100 10]; % just assume something ;-)
        elseif length(glo.opt) == 2 % swarm, but some info is missing
            glo.opt(3) = 10; % just assume something ;-)
        end
    elseif glo.opt(1) == 2 && length(glo.opt) == 1 % annealing, but some info missing
        glo.opt(2) = 1e-4; % default temperature
    elseif glo.opt(1) == 1 && length(glo.opt) == 1 % simplex, but some info missing
        glo.opt(2) = 2; % default two runs
    end
end

if ~isfield(glo,'plt')
    glo.plt = [0 0 0]; % by default, plot in colour
elseif length(glo.plt) == 1
    glo.plt([2 3]) = [1 0]; % by default, plot connecting line
elseif length(glo.plt) == 2
    glo.plt(3) = 0; % by default, do not limit axes to data
end

% =========================================================================
%                       PRELIMINARY CHECKS ON DATA
% =========================================================================

for i = 1:ndata % run through state variables
    if size(W,2) >= i
        sD = size(DATA{i}); % calculate size of data set i
        sW = size(W{i});   % calculate size of weight factors for set i
        if sD(1) == sW(1) && sD(2) == sW(2) % if they are exact same size ...
            W{i} = W{i}(2:end,2:end); % assume that time and scenario where added to weights, so trim them off
            sW = size(W{i});   % calculate size of weight factors for set i again
        end
        if sum(sW) ~= 0 && (sD(1)-1 ~= sW(1) || sD(2)-1 ~= sW(2)) % if they NOT are exact same size ...
            error(['in data set ',num2str(i),' the weight matrix does not match the data set.'])
        end
    end
    if DATA{i}(1,1) == -1 % if we have survival data ...
        a = DATA{i}(2:end,2:end); % extract data set
        b = diff(a); % deaths in each interval
        if any(b(:)>0) % make sure there are no zombies in there ...
            error(['in data set ',num2str(i),' number of survivors should never increase in time.'])
        end
        if any(X0mat(i+1,:) > 1) % and check that the initial values are reasonable
            error(['for state number ',num2str(i),' initial values in X0mat should be probabilities (so <1).'])
        end
    end
end

if ~isempty(glo) && isfield(glo,'wts') % check if weights are right size
    if sum(size(DATA) == size(glo.wts)) ~= 2 % sizes do not match
        error(['Weights matrix glo.wts must be same size as data sets matrix DATA: ',num2str(n_D),'x',num2str(n_X)])
    end
end
if ~isempty(glo) && isfield(glo,'var') % check if variances are right size
    if sum(size(DATA) == size(glo.var)) ~= 2 % sizes do not match
        error(['Variance matrix glo.var must be same size as data sets matrix DATA: ',num2str(n_D),'x',num2str(n_X)])
    end
end

% Make sure that weight matrix is specified correctly
add_wts = 0; % keep track whether weights are added
for i = 1:ndata % loop over all data sets
    if isempty(W) || length(W(:))<i || isempty(W{i}) || numel(W{i}) < 2 % if weights are not specified for this data set ...
        if DATA{i}(1,1) == -1 % if we have survival data ...
            W{i} = zeros(size(DATA{i})-1); % make zeros with the same size as the data (without times and scenarios)
            % Note that for survival, the weights matrix specifies missing/removed animals
            add_wts = 1;
        else
            W{i} = ones(size(DATA{i})-1); % make ones with the same size as the data (without times and scenarios)
            add_wts = 1;
        end
        if sum(size(W{i}) == size(DATA{i})-1) ~= 2 % size of weight matrix does not match data set
            error(['Weight matrix of data set ',num2str(i),' is not the size of the data set.'])
        end
    end
end
if n_D > 1 && add_wts == 1 % weights have been added, so a reshape might be needed
    W = reshape(W,n_D,n_X);
end

% =========================================================================
%                       PRELIMINARY CHECKS ON EXTRA DATA
% =========================================================================

% we also need to check DATAx and Wx, but for now, fix Wx as done for W
if n_X2 > 0
    for i = 1:n_X2 % run through extra data sets
        if isempty(DATAx{i}) % catch cases where data were forgotten ...
            DATAx{i} = 0;
        end
        if size(Wx,2) >= i
            sD = size(DATAx{i}); % calculate size of data set i
            sW = size(Wx{i});   % calculate size of weight factors for set i
            if sD(1) == sW(1) && sD(2) == sW(2) % if they are exact same size ...
                Wx{i} = Wx{i}(2:end,2:end); % assume that time and scenario where added to weights, so trim them off
                sW = size(Wx{i});   % calculate size of weight factors for set i again
            end
            if sum(sW) ~= 0 && (sD(1)-1 ~= sW(1) || sD(2)-1 ~= sW(2)) % if they NOT are exact same size ...
                error(['in extra data set ',num2str(i),' the weight matrix does not match the data set.'])
            end
        end
    end
    % Make sure that weight matrix is specified correctly
    add_wts = 0; % keep track whether weights are added
    for i = 1:n_X2 % loop over all extra data sets
        if isempty(Wx) || length(Wx(:))<i || isempty(Wx{i}) || numel(Wx{i}) < 2 % if weights are not specified for this data set ...
            Wx{i} = ones(size(DATAx{i})-1); % make ones with the same size as the data (without times and scenarios)
            add_wts = 1;
            if sum(size(Wx{i}) == size(DATAx{i})-1) ~= 2 % size of weight matrix does not match data set
                error(['Weight matrix of extra data set ',num2str(i),' is not the size of the data set.'])
            end
        end
    end
end

% if axis and legend labels are forgotten, just give default names
if ~exist('ylab2') || isempty(ylab2{1})
    ylab2{1} = 'Y-values extra data 1';
end
if n_X2 ~= size(ylab2,2)
    for i = 2:n_X2
        if length(ylab2) < i
            ylab2{i} = ['Y-values extra data ',num2str(i)];
        end
    end
end
if ~exist('xlab2') || isempty(xlab2{1})
    xlab2{1} = 'X-values extra data 1';
end
if n_X2 ~= size(xlab2,2)
    for i = 2:n_X2
        if length(xlab2) < i
            xlab2{i} = ['X-values extra data ',num2str(i)];
        end
    end
end

% =========================================================================
%                       OTHER CHECKS ON INPUTS
% =========================================================================

names  = fieldnames(par); % extract all field names of par (global)

% Create time and concentration vectors for fitting. This is now done here,
% always, because that allows the user to make a profile likelihood after a
% simulation.
tvect = []; % initialise time vector as empty matrix
cvect = []; % initialise concentration vector as empty matrix
for i = 1:ndata,
    tvect = [glo.Tinit;tvect;DATA{i}(2:end,1)];  % append all time vectors from the data set
    % and make sure that the initial time is in there!
    cvect = [cvect;(DATA{i}(1,2:end))']; % append all concentration vectors from the data set
    lam = DATA{i}(1,1); % how to transform the data
    if isnan(lam) % if some smartass puts a NaN there ...
        error(['The first number in the data matrix for state ',num2str(i),' should be a number. Check the byom script for the meaning of that number.'])
    elseif (lam < 0 && lam ~= -1) || lam > 2
        error(['The first number in the data matrix for state ',num2str(i),' cannot be interpreted. Check the byom script for the meaning of that number.'])
    end
end
if isfield(glo,'sel') && glo.sel ~= 1
    % for the GUTS packages, we sometimes need a long time vector
    % for IT, we have to find the maximum of the dose metric
    % for mixed, we cumulate the hazard rate here based on the dose metric
    tvect = [tvect;(linspace(min(tvect),max(tvect),100))'];
end
ttot = unique(tvect(tvect>=glo.Tinit)); % total vector with unique time points (global)

if all(tvect == glo.Tinit) && fit ~= 0
    error('All timepoints in the dataset are for the initial time point, for simulation only: set fit=0 in script!')
end

% some adaptations if the time vector of the data is really short!
if length(ttot) == 2,
    ttot = [ttot(1) mean(ttot) ttot(2)];
end

ctot = X0mat(1,:);    % this is the concentration vector that will be used!
% Note: the model is run ONLY with the concentrations/scenarios that are
% given in the initial state matrix X0mat! Scenarios that are in the data
% set but not in X0mat are thus ignored.

if length(ctot) ~= length(unique(ctot))
    error('The scenario numbers in X0mat (first row) must be unique!')
end

% =========================================================================
%                       PRELIMINARY CHECKS ON PARAMETERS
% =========================================================================

if fit == 1 % if we don't fit, there is no need to care about the parameter vector
    for i = 1:length(names) % run through all parameters
        a = eval(['par.',names{i}]);
        if length(a) < 4
            error('Check your parameter definition: specify a vector with 4-5 values for each parameter.')
        end
        if length(a) == 4
            a(5) = 1; % add a one
            eval(['par.',names{i},'=a;']);
        end
        if a(5) == 0 % look at log-fitted parameters
            if a(1) == 0 % initial value is zero
                error('You fit one (or more) parameters on log scale and have an initial value of zero!')
            end
            if a(3) == 0 % lower bound is zero
                eval(['par.',names{i},'(3)=1e-10;']); % simply assume 1e-10 is low enough ...
            end
        end
    end
end

pmat   = packunpack(1,par,0);  % transform structure into a regular matrix
% if any parameter is outside the bounds, error
if any(pmat(:,1)<pmat(:,3) | pmat(:,1)>pmat(:,4)),
    error('Initial value of one or more parameters is out of min-max bounds!')
end
        
    


