% Script to simulate and plot (systems of) ODEs 'real time'
% Various plotting options are available
% 1) 3d-plot (specify which 3 states, in which order, in your byom script)
% 2) 2d-plot (specify which 2 states, in which order, in your byom script)
% 3) subplots for each state (scenarios as different color)
% 4) subplots for each scenario (states are different color)
% 5) dx/dt versus x for each state in a separate subplot
% 6) 2d plot for each scenario separate
% 7) 3d plot for each scenario separate

% Author : Tjalling Jager 
% Date: April 2015
% Web support: http://www.debtox.info/byom.html

c  = X0mat(1,:);      % scenarios
ns = size(X0mat,1)-1; % number of states
nc = length(c);       % number of scenarios

plotcol  = 'krbgcmy'; % 7 colors should be enough ...
trainsz  = 8; % plot size of the running 'train'
trailsz  = 1; % plot size of the permanent 'trail' (zero for NO trail)
plotstrt = 1; % plot starting point in 2d and 3d plots
plotvct  = 0; % plot field of vector arrows in 2d plots when possible

% =========================================================================
% some preliminary checks ...
if ~exist('plottype')
    plottype = 4;
end
if ns > 7 && plottype == 4
    error('Too many states for this plottype')
end
if nc > 7 && plottype == 3
    error('Too many scenarios for this plottype')
end
if ns < 3 && plottype == 1
    error('Not enough state variable for a 3d plot')
end
if ns < 2 && (plottype == 2 || plottype == 6)
    error('Not enough state variable for a 2d plot')
end
if ~exist('plot_int')
    plot_int = 10;
else
    plot_int = max(3,plot_int); % less than 3 gives problems with calling call_deri
end
if ~exist('stx')
    stx = 1;
end
if ~exist('sty') && ns > 1
    sty = 2;
end
if ~exist('stz') && ns > 2
    stz = 3;
end
if ~exist('ylab') || isempty(ylab{1})
    ylab{1} = 'State var. 1';
end
if ns ~= size(ylab,2)
    for i = 2:ns
        if length(ylab) < i
            ylab{i} = ['State var. ',num2str(i)];
        end
    end
end
if ~exist('xlab')
    xlab = 'time';
end
if ~exist('leglab1')
    leglab1 = 'Scen. ';
end
if ~exist('leglab2')
    leglab2 = '';
end
% =========================================================================

hd = figure;
hold on
axis square
figure(hd) % bring figure to front

switch plottype
    case 1
        
        % =================================================================
        % 3D plot
        % =================================================================
        view(-20,25) % camera angle
        grid on
        xlabel(ylab{stx},'FontSize',12)
        ylabel(ylab{sty},'FontSize',12)
        zlabel(ylab{stz},'FontSize',12)
        title(['Plotted from: ',mfilename_base, ' (',date,')'],'Interpreter', 'none')
        
        if exist('axrange') % if there is an axis range, use it
            axis([axrange(stx,:) axrange(sty,:) axrange(stz,:)])
        end
        if exist('Xeq') % if equilibria are given, plot them
            for ieq = 1:size(Xeq,1)
                stem3(Xeq(ieq,stx),Xeq(ieq,sty),Xeq(ieq,stz),'ko','MarkerFaceColor','y','BaseValue',axrange(stz,1),'LineWidth',2)
                % plot3(Xeq(ieq,stx),Xeq(ieq,sty),Xeq(ieq,stz),'ko','MarkerFaceColor','y')
            end
        end
        
        X0tmp = X0mat;
        Tn = t(1:plot_int);
        
        % First run to get started
        for k = 1:nc % run through scenarios
            if plotstrt == 1
                X0 = X0mat(2:end,k); % extract starting values for this scenario
                plot3(X0(stx),X0(sty),X0(stz),[plotcol(k),'s'],'MarkerFaceColor',plotcol(k)) % plot starting point
            end
            X = call_deri(Tn,par,X0mat(:,k)); % use call_deri.m to provide the output
            hl1(k)=plot3(X(:,stx),X(:,sty),X(:,stz),[plotcol(k),'-'],'LineWidth',trainsz);
            if trailsz > 0
                plot3(X(:,stx),X(:,sty),X(:,stz),[plotcol(k),'-'],'LineWidth',trailsz);
            end
            drawnow
            X0tmp(2:end,k) = (X(end,:))';
            L{k} = [leglab1,num2str(X0mat(1,k)),leglab2]; % create legend entries
        end
        legend(hl1,L) % plot legend using plot handles
        pause(0.05) % strangely enough, otherwise the legend is ruined ...
        
        % Continue from the first run for the remainder of the time vector
        for i = 2:floor(length(t)/plot_int) % note that some points might not be plotted now!
            for k = 1:nc
                Tn = t((i-1)*plot_int:i*plot_int); % new time vector
                X = call_deri(Tn,par,X0tmp(:,k)); % use call_deri.m to provide the output
                delete(hl1(k)) % delete old plot segment before plotting a new one
                hl1(k) = plot3(X(:,stx),X(:,sty),X(:,stz),[plotcol(k),'-'],'LineWidth',trainsz);
                if trailsz > 0
                    plot3(X(:,stx),X(:,sty),X(:,stz),[plotcol(k),'-'],'LineWidth',trailsz);
                end
                drawnow
                X0tmp(2:end,k) = (X(end,:))'; % use previous last as new first
            end
        end
        if trailsz > 0
            for k = 1:nc
                delete(hl1(k)) % delete old plot segment at the end
            end
        end
        
    case 2
        
        % =================================================================
        % 2D plot
        % =================================================================
        
        if exist('Xeq')
            for ieq = 1:size(Xeq,1)
                plot(Xeq(ieq,stx),Xeq(ieq,sty),'ko','MarkerFaceColor','y')
            end
        end
        xlabel(ylab(stx),'FontSize',12)
        ylabel(ylab{sty},'FontSize',12)
        title(['Plotted from: ',mfilename_base, ' (',date,')'],'Interpreter', 'none')
        
        X0tmp = X0mat;
        Tn = t(1:plot_int);
        
        if exist('axrange') % if there is an axis range, use it
            axis([axrange(stx,:) axrange(sty,:)])
            
            % if there is an axis range, we can also make a vector field!
            if plotvct == 1 && ns == 2 && nc == 1 % only when there are 2 states and 1 scenario
                % if there are more scenarios in one plot, this is too messy
                % (I cannot plot more than 1 vector field in a plot)
                x_grid = linspace(axrange(stx,1),axrange(stx,2),15);
                y_grid = linspace(axrange(sty,1),axrange(sty,2),15);
                for i =1:length(x_grid)
                    for j = 1:length(y_grid)
                        X0tmp(stx) = x_grid(i);
                        X0tmp(sty) = y_grid(j);
                        dXtmp = derivatives(0,X0tmp,par,c(1)); % use derivatives to calculate dX/dt
                        dX(i,j) = dXtmp(stx);
                        dY(i,j) = dXtmp(sty);
                    end
                end
                [Xpl,Ypl] = meshgrid(x_grid,y_grid);
                quiver(Xpl,Ypl,dX',dY',1.5)
            end
        end
        
        % First run to get started
        for k = 1:nc
            if plotstrt == 1
                X0 = X0mat(2:end,k);
                plot(X0(stx),X0(sty),[plotcol(k),'s','MarkerFaceColor',plotcol(k)]) % plot start point
            end
            X = call_deri(Tn,par,X0mat(:,k)); % use call_deri.m to provide the output
            hl1(k)=plot(X(:,stx),X(:,sty),[plotcol(k),'-'],'LineWidth',trainsz);
            if trailsz > 0
                plot(X(:,stx),X(:,sty),[plotcol(k),'-'],'LineWidth',trailsz);
            end
            drawnow
            X0tmp(2:end,k) = (X(end,:))';
            L{k} = [leglab1,num2str(X0mat(1,k)),leglab2]; % create legend entries
        end
        legend(hl1,L) % plot legend using plot handles
        
        % Continue from the first run for the remainder of the time vector
        for i = 2:floor(length(t)/plot_int) % note that some points might not be plotted now!
            for k = 1:nc
                Tn = t((i-1)*plot_int:i*plot_int); % new time vector
                X = call_deri(Tn,par,X0tmp(:,k)); % use call_deri.m to provide the output
                delete(hl1(k)) % delete old plot segment before plotting a new one
                hl1(k) = plot(X(:,stx),X(:,sty),[plotcol(k),'-'],'LineWidth',trainsz);
                if trailsz > 0
                    plot(X(:,stx),X(:,sty),[plotcol(k),'-'],'LineWidth',trailsz);
                end
                drawnow
                X0tmp(2:end,k) = (X(end,:))'; % use previous last as new first
            end
        end
        if trailsz > 0
            for k = 1:nc
                delete(hl1(k)) % delete old plot segment at the end
            end
        end
        
    case 3
        
        % =================================================================
        % make one subplot for each state variable
        % =================================================================
        n = ceil(sqrt(ns));
        m = ceil(ns/n);
        for j = 1:ns % run through states
            subplot(m,n,j)
            hold on
            xlabel(xlab,'FontSize',12)
            ylabel(ylab{j},'FontSize',12)
            title(['Plotted from: ',mfilename_base, ' (',date,')'],'Interpreter', 'none')
            
            if exist('axrange') % if there is an axis range, use it
                axis([0 t(end) axrange(j,:)])
            else
                xlim([0 t(end)])
            end
        end
        
        X0tmp = X0mat;
        Tn = t(1:plot_int);
        
        % First run to get started
        for k = 1:nc
            X = call_deri(Tn,par,X0mat(:,k)); % use call_deri.m to provide the output
            for j = 1:ns
                subplot(m,n,j)
                hold on
                plot(Tn,X(:,j),[plotcol(k),'-'],'LineWidth',2);
                L{k} = [leglab1,num2str(X0mat(1,k)),leglab2];
            end
            drawnow
            X0tmp(2:end,k) = (X(end,:))';
        end
        for j = 1:ns % create legends in each subplot
            subplot(m,n,j)
            legend(L,'FontSize',12)
        end
        
        % Continue from the first run for the remainder of the time vector
        for i = 2:floor(length(t)/plot_int) % note that some points might not be plotted now!
            for k = 1:nc
                Tn = t((i-1)*plot_int:i*plot_int); % new time vector
                X = call_deri(Tn,par,X0tmp(:,k)); % use call_deri.m to provide the output
                for j = 1:ns
                    subplot(m,n,j)
                    hold on
                    plot(Tn,X(:,j),[plotcol(k),'-'],'LineWidth',2);
                end
                drawnow
                X0tmp(2:end,k) = (X(end,:))'; % use previous last as new first
            end
        end
        
    case 4
        
        % =================================================================
        % make one plot for each scenario
        % =================================================================
        n = ceil(sqrt(nc));
        m = ceil(nc/n);
        for k = 1:nc % run through scenarios
            subplot(m,n,k)
            hold on
            xlabel(xlab,'FontSize',12)
            ylabel(['state variables, ',leglab1,num2str(c(k)),leglab2],'FontSize',12)
            title(['Plotted from: ',mfilename_base, ' (',date,')'],'Interpreter', 'none')
            
            if exist('axrange') % if there is an axis range, use it
                axis([0 t(end) min(axrange(:)) max(axrange(:))])
            else
                xlim([0 t(end)])
            end
        end
        
        X0tmp = X0mat;
        Tn = t(1:plot_int);
        
        % First run to get started
        for k = 1:nc
            X = call_deri(Tn,par,X0mat(:,k)); % use call_deri.m to provide the output
            subplot(m,n,k)
            hold on
            for j = 1:ns
                plot(Tn,X(:,j),[plotcol(j),'-'],'LineWidth',2);
                L{j} = [ylab{j}];
            end
            drawnow
            X0tmp(2:end,k) = (X(end,:))';
        end
        for k = 1:nc % create legends for each subplot
            subplot(m,n,k)
            legend(L,'FontSize',12)
        end
        
        % Continue from the first run for the remainder of the time vector
        for i = 2:floor(length(t)/plot_int) % note that some points might not be plotted now!
            for k = 1:nc
                Tn = t((i-1)*plot_int:i*plot_int); % new time vector
                X = call_deri(Tn,par,X0tmp(:,k)); % use call_deri.m to provide the output
                subplot(m,n,k)
                hold on
                for j = 1:ns
                    plot(Tn,X(:,j),[plotcol(j),'-'],'LineWidth',2);
                end
                drawnow
                X0tmp(2:end,k) = (X(end,:))'; % use previous last as new first
            end
        end
        
    case 5
        
        % =================================================================
        % dx/dt vs x; one subplot for each state variable
        % =================================================================
        n = ceil(sqrt(ns));
        m = ceil(ns/n);
        for j = 1:ns % run through states
            subplot(m,n,j)
            hold on
            xlabel(ylab{j},'FontSize',12)
            ylabel(['d',ylab{j},'/dt'],'FontSize',12)
            
            title(['Plotted from: ',mfilename_base, ' (',date,')'],'Interpreter', 'none')
            
            if exist('axrange') % if there is an axis range, use it
                xlim([axrange(j,:)]) % only limit X range
                plot(axrange(j,:),[0 0],'k:')
            end
            if exist('Xeq')
                for ieq = 1:size(Xeq,1)
                    plot(Xeq(ieq,j),0,'ko','MarkerFaceColor','y')
                end
            end
        end
        
        X0tmp = X0mat;
        Tn = t(1:plot_int);
        
        % First run to get started
        for k = 1:nc
            X = call_deri(Tn,par,X0mat(:,k)); % use call_deri.m to provide the output
            for ki =1:plot_int % run through time vector so that derivatives can stay the same
                dX(ki,:) = (derivatives(Tn(ki),(X(ki,:))',par,c(k)))'; % use derivatives to calculate dX/dt
            end
            for j = 1:ns
                subplot(m,n,j)
                hold on
                hl1(k,j) = plot(X(:,j),dX(:,j),[plotcol(k),'-'],'LineWidth',2);
                L{k} = [leglab1,num2str(X0mat(1,k)),leglab2];
            end
            drawnow
            X0tmp(2:end,k) = (X(end,:))';
        end
        for j = 1:ns % create legends in each subplot
            subplot(m,n,j)
            legend(hl1(:,j),L,'FontSize',12)
        end
        
        % Continue from the first run for the remainder of the time vector
        for i = 2:floor(length(t)/plot_int) % note that some points might not be plotted now!
            for k = 1:nc
                Tn = t((i-1)*plot_int:i*plot_int); % new time vector
                X = call_deri(Tn,par,X0tmp(:,k)); % use call_deri.m to provide the output
                for ki =1:plot_int+1 % run through time vector so that derivatives can stay the same
                    dX(ki,:) = (derivatives(Tn(ki),(X(ki,:))',par,c(k)))'; % use derivatives to calculate dX/dt
                end
                
                for j = 1:ns
                    subplot(m,n,j)
                    hold on
                    plot(X(:,j),dX(:,j),[plotcol(k),'-'],'LineWidth',2);
                end
                drawnow
                X0tmp(2:end,k) = (X(end,:))'; % use previous last as new first
            end
        end
        
    case 6
        
        % =================================================================
        % make one 2d plot for each scenario
        % =================================================================
        n = ceil(sqrt(nc));
        m = ceil(nc/n);
        for k = 1:nc % run through scenarios
            subplot(m,n,k)
            hold on
            xlabel(ylab(stx),'FontSize',12)
            ylabel(ylab{sty},'FontSize',12)
            title(['Plotted from: ',mfilename_base, ' (',date,')'],'Interpreter', 'none')
            
            if exist('Xeq')
                for ieq = 1:size(Xeq,1)
                    plot(Xeq(ieq,stx),Xeq(ieq,sty),'ko','MarkerFaceColor','y')
                end
            end
            
            if exist('axrange') % if there is an axis range, use it
                axis([axrange(stx,:) axrange(sty,:)])
                % if there is an axis range, we can also make a vector field!
                if plotvct == 1 && ns == 2 % only when there are 2 states
                    x_grid = linspace(axrange(stx,1),axrange(stx,2),15);
                    y_grid = linspace(axrange(sty,1),axrange(sty,2),15);
                    for i =1:length(x_grid)
                        for j = 1:length(y_grid)
                            X0tmp(stx) = x_grid(i);
                            X0tmp(sty) = y_grid(j);
                            dXtmp = derivatives(0,X0tmp,par,c(k)); % use derivatives to calculate dX/dt
                            dX(i,j) = dXtmp(stx);
                            dY(i,j) = dXtmp(sty);
                        end
                    end
                    [Xpl,Ypl] = meshgrid(x_grid,y_grid);
                    quiver(Xpl,Ypl,dX',dY',1.5)
                end
            end
        end
        
        X0tmp = X0mat;
        Tn = t(1:plot_int);
        
        % First run to get started
        for k = 1:nc
            subplot(m,n,k)
            hold on
            if plotstrt == 1
                X0 = X0mat(2:end,k);
                plot(X0(stx),X0(sty),[plotcol(k),'s'],'MarkerFaceColor',plotcol(k))
            end
            
            X = call_deri(Tn,par,X0mat(:,k)); % use call_deri.m to provide the output
            subplot(m,n,k)
            hold on
            hl1(k)=plot(X(:,stx),X(:,sty),[plotcol(k),'-'],'LineWidth',trainsz);
            if trailsz > 0
                plot(X(:,stx),X(:,sty),[plotcol(k),'-'],'LineWidth',trailsz);
            end
            drawnow
            X0tmp(2:end,k) = (X(end,:))';
            legend(hl1(k),[leglab1,num2str(X0mat(1,k)),leglab2]); % create legend entries)
            
        end
        
        % Continue from the first run for the remainder of the time vector
        for i = 2:floor(length(t)/plot_int) % note that some points might not be plotted now!
            for k = 1:nc
                Tn = t((i-1)*plot_int:i*plot_int); % new time vector
                X = call_deri(Tn,par,X0tmp(:,k)); % use call_deri.m to provide the output
                subplot(m,n,k)
                hold on
                delete(hl1(k)) % delete old plot segment before plotting a new one
                hl1(k)=plot(X(:,stx),X(:,sty),[plotcol(k),'-'],'LineWidth',trainsz);
                if trailsz > 0
                    plot(X(:,stx),X(:,sty),[plotcol(k),'-'],'LineWidth',trailsz);
                end
                drawnow
                X0tmp(2:end,k) = (X(end,:))'; % use previous last as new first
            end
        end
        if trailsz > 0
            for k = 1:nc
                delete(hl1(k)) % delete old plot segment at the end
            end
        end
        
    case 7
        
        % =================================================================
        % make one 3d plot for each scenario
        % =================================================================
        n = ceil(sqrt(nc));
        m = ceil(nc/n);
        for k = 1:nc % run through scenarios
            subplot(m,n,k)
            hold on
            view(-20,25) % camera angle
            grid on
            xlabel(ylab(stx),'FontSize',12)
            ylabel(ylab{sty},'FontSize',12)
            zlabel(ylab{stz},'FontSize',12)
            title(['Plotted from: ',mfilename_base, ' (',date,')'],'Interpreter', 'none')
            
            if exist('Xeq')
                for ieq = 1:size(Xeq,1)
                    stem3(Xeq(ieq,stx),Xeq(ieq,sty),Xeq(ieq,stz),'ko','MarkerFaceColor','y','BaseValue',axrange(stz,1),'LineWidth',2)
                end
            end
            
            if exist('axrange') % if there is an axis range, use it
                axis([axrange(stx,:) axrange(sty,:) axrange(stz,:)])
            end
        end
        
        X0tmp = X0mat;
        Tn = t(1:plot_int);
        
        % First run to get started
        for k = 1:nc
            subplot(m,n,k)
            hold on
            if plotstrt == 1
                X0 = X0mat(2:end,k);
                plot3(X0(stx),X0(sty),X0(stz),[plotcol(k),'s'],'MarkerFaceColor',plotcol(k)) % plot starting point
            end
            
            X = call_deri(Tn,par,X0mat(:,k)); % use call_deri.m to provide the output
            subplot(m,n,k)
            hold on
            hl1(k)=plot3(X(:,stx),X(:,sty),X(:,stz),[plotcol(k),'-'],'LineWidth',trainsz);
            if trailsz > 0
                plot3(X(:,stx),X(:,sty),X(:,stz),[plotcol(k),'-'],'LineWidth',trailsz);
            end
            drawnow
            X0tmp(2:end,k) = (X(end,:))';
            legend(hl1(k),[leglab1,num2str(X0mat(1,k)),leglab2]); % create legend entries)
            
        end
        
        % Continue from the first run for the remainder of the time vector
        for i = 2:floor(length(t)/plot_int) % note that some points might not be plotted now!
            for k = 1:nc
                Tn = t((i-1)*plot_int:i*plot_int); % new time vector
                X = call_deri(Tn,par,X0tmp(:,k)); % use call_deri.m to provide the output
                subplot(m,n,k)
                hold on
                delete(hl1(k)) % delete old plot segment before plotting a new one
                hl1(k) = plot3(X(:,stx),X(:,sty),X(:,stz),[plotcol(k),'-'],'LineWidth',trainsz);
                if trailsz > 0
                    plot3(X(:,stx),X(:,sty),X(:,stz),[plotcol(k),'-'],'LineWidth',trailsz);
                end
                drawnow
                X0tmp(2:end,k) = (X(end,:))'; % use previous last as new first
            end
        end
        if trailsz > 0
            for k = 1:nc
                delete(hl1(k)) % delete old plot segment at the end
            end
        end
end


