function [par_out,FVAL] = start_calc(par,filenm,it)

% This function starts up the optimisation of the parameter values. It
% calls the Matlab function fminsearch which performs a Nelder-Mead Simplex
% search. This file also display the results in a formatted manner.
%
% The parameter structure <par> is input, output is the structure <par_out>
% which contains the optimised values in the first position. 
% 
% Author : Tjalling Jager 
% Date: July 2015
% Web support: http://www.debtox.info/byom.html

global DATA names namesp pri zvd namesz glo n_X n_D

ndata = n_X * n_D; % total number of data sets

% =========================================================================
% preliminary checks are now done in the script prelim_checks.m, which is
% called in calc_and_plot.m.
% =========================================================================

if ~isempty(pri)
    namesp = fieldnames(pri);  % extract all field names of prior (global)
end
if ~isempty(zvd)
    namesz = fieldnames(zvd);  % extract all field names of zero-var data (global)
end

pmat   = packunpack(1,par,0);  % transform structure into a regular matrix
% put parameters that need to be on log scale on log10 scale
pmat(pmat(:,5)==0,1) = log10(pmat(pmat(:,5)==0,1));
pfit   = pmat(pmat(:,2)==1,1); % parameter values that are to be estimated

oldopts    = optimset('fminsearch'); % default options for fminsearch
options_s1 = optimset(oldopts,'FunValCheck','on'); % function check on
if it == 1 % show iterations
    options_s1 = optimset(options_s1,'Display','iter','FunValCheck','on'); % show all iterations
end
options_s2 = optimset(options_s1,'TolX',1e-2,'TolFun',1e-2,'MaxFunEvals',30*length(pfit)); % rough Simplex options

switch glo.opt(1)
    case 1 % standard two simplexes
        [phat,FVAL,EXITFLAG,OUTPUT] = fminsearch('transfer',pfit,options_s2,pmat); % first rough estimation
        out_iter = OUTPUT.iterations; % remember iterations
        for no_opt = 1:glo.opt(2)-1        
            [phat,FVAL,EXITFLAG,OUTPUT]  = fminsearch('transfer',phat,options_s1,pmat); % than a more detailed one
            out_iter = out_iter+OUTPUT.iterations; % count total iterations
        end
    case 2 % simulated annealing + simplex
        opt_ann.Verbosity = 2; % controls output on screen
        opt_ann.StopTemp  = glo.opt(2); % crude temperature for stopping as Simplex will finish it off
        [phat,FVAL] = anneal(@transfer,pfit,opt_ann,pmat); % first rough estimation
        [phat,FVAL,EXITFLAG,OUTPUT]  = fminsearch('transfer',phat,options_s1,pmat); % followed by a detailed simplex
        out_iter = OUTPUT.iterations; % remember iterations
    case 3 % particle swarm + simplex
        % set parameters based on dimensionality of the problem, and width of min-max bounds!
        nobirds = glo.opt(2); % number of birds in the swarm
        noiter  = glo.opt(3); % number of iterations (don't need too many as Simplex will follow)
        swarm_bnds = pmat(:,[3 4]); 
        swarm_bnds(pmat(:,5)==0,:) = log10(swarm_bnds(pmat(:,5)==0,:)); % if parameters are estimated on log-scale, the bounds must also be log scale
        swarm_bnds = swarm_bnds(pmat(:,2)==1,:); % only fitted parameters
        phat = Particle_Swarm_Optimization (nobirds,length(pfit),swarm_bnds,@transfer,'min',2,2,2,0.4,0.9,noiter,pmat);
        out_iter = 0; % do not count swarm iterations in noiter
        [phat,FVAL,EXITFLAG,OUTPUT]  = fminsearch('transfer',phat,options_s1,pmat); % followed by a detailed simplex
        out_iter = out_iter+OUTPUT.iterations; % remember iterations
end

[minloglik,GoodFit] = transfer(phat,pmat); % get goodness of fit measures
disp('')
disp('Provisional goodness-of-fit measures for each data set')
disp(GoodFit)
% GoodFit is a vector with goodness-of-fit measures for each data set. For
% continuous data, it is the R-squared (non-adjusted). For survival data it
% is the p-value for a likelihood-ratio test between the best fit and the
% best hypothetical fit (giving every interval its own death probability),
% according to bedaux & Kooijman (1994). For multi-fits, this p-value
% probably becomes less meaningfull ... 

% minimisation is done with the reduced parameter set, other parameters in par are kept
% to the fixed value. This means that phat will also contain the limited set only.
% To make the vector whole again:
p0 = pmat(:,1);          % take the first column with the initial parameter values
p = p0;                  % copy the initial values to a new vector
p(pmat(:,2)==1) = phat ; % replace values in total vector with values that were fitted

% put parameters that need to be fitted on log scale back on normal scale
p(pmat(:,5)==0) = 10.^(p(pmat(:,5)==0));
p0(pmat(:,5)==0) = 10.^(p0(pmat(:,5)==0));

p = max(p,pmat(:,3));    % force values within the specified lower parameter bounds
p = min(p,pmat(:,4));    % force values within the specified upper parameter bounds
pmat(:,1) = p;           % return the new p vector into the matrix pmat

par_out = packunpack(2,0,pmat); % transform matrix into a structure to return to the script

% ========== Now display the results on screen ==========

diary ('results.out') % collect output in the diary "results.out"
disp(' ')
disp('=================================================================================')
disp('Results of the parameter estimation with BYOM version 3.3')
disp('=================================================================================')
fprintf('   Filename      : %s \n',filenm) % name of the mydata file
ti = clock; % what is the current time?
fprintf('   Analysis date : %s (%02.0f:%02.0f) \n',date,ti(4),ti(5));

disp('   Data entered  :')
for i = 1:n_X % run through states
    for j = 1:n_D % run through number of data sets per state
        lam = DATA{j,i}(1,1); % likelihood type
        datsiz = size(DATA{j,i})-1;
        if n_D == 1 % if there is only one data set ...
            fprintf('     data state %1.0f: %1.0fx%1.0f, ',i,datsiz(1),datsiz(2))
        else % if there's more, also print the number of the data set between parentheses
            fprintf('     data state %1.0f(%1.0f): %1.0fx%1.0f, ',i,j,datsiz(1),datsiz(2))
        end
        switch lam
            case -1
                fprintf('survival data.')
            case 0
                if prod(datsiz) == 0
                    fprintf('no data.')
                else
                    fprintf('continuous data, log transf.')
                end
            case 1
                fprintf('continuous data, no transf.')
            otherwise
                fprintf('continuous data, power %1.2f transf.',lam)
        end
        if isfield(glo,'wts') % weights have been applied
            fprintf(' Weight: %1.4g.',glo.wts(i))
        end
        if isfield(glo,'var') % variances have been supplied
            fprintf(' Var: %1.4g.',glo.var(i))
        end
        fprintf('\n')
    end
end

switch glo.opt(1)
    case 1
        fprintf('   Search method: Nelder-Mead simplex direct search, %g rounds. \n',glo.opt(2))
    case 2
        fprintf('   Search method: Simulated annealing plus simplex search, stop temp. %g. \n',glo.opt(2))
    case 3
        fprintf('   Search method: Swarm optimisation plus simplex search, %g particles, %g iter. \n',glo.opt(2),glo.opt(3))
end        

if EXITFLAG == 1,
    disp('     The optimisation routine has converged to a solution')
else
    disp('     No convergence, optimisation stopped on maximum number of iterations')
end
fprintf('     Total %1.0f simplex iterations used to optimise. \n',out_iter)
fprintf('     Minus log-likelihood has reached the value %1.6g. \n',FVAL)

disp('=================================================================================')
nfields = length(names);
for i = 1:nfields, % display all parameters on screen
    fprintf('%-6s %10.4g (fit: %1.0f, initial: %2.4g) ',names{i},p(i),pmat(i,2),p0(i))
    if ~isempty(pri)
        if isfield(pri,names{i}) % is there a prior for this one?
            a = eval(['pri.',names{i}]); % extract prior info
            switch a(1)
                case 1
                    fprintf('\t prior: beta (%g,%g)',a(2),a(3))
                case 2
                    fprintf('\t prior: tri  (%g-%g-%g)',a(2),a(4),a(3))
                case 3
                    fprintf('\t prior: norm (mn %g,sd %g)',a(2),a(3))
                case 4
                    fprintf('\t prior: logn (lnmn %g,lnsd %g)',a(2),a(3))
                otherwise
                    fprintf('\t prior: unknown distribution ...')
            end
        else
            fprintf('\t prior: unif (%g-%g)',pmat(i,3),pmat(i,4))
        end
    end
    fprintf('\n')
end
if any(pmat(:,5)==0)
    disp('=================================================================================')
    ind_log = find(pmat(:,5)==0); % find parameters that are on log scale
    if length(ind_log)==1
        fprintf('Parameter ')
    else
        fprintf('Parameters ')
    end
    for i = 1:length(ind_log)
        fprintf('%s',names{ind_log(i)})
        if i ~= length(ind_log)
            if i == length(ind_log)-1
                fprintf(' and ')
            else
                fprintf(', ')
            end
        end
    end
    if length(ind_log)==1
        fprintf(' is fitted on log-scale.\n')
    else
        fprintf(' are fitted on log-scale.\n')
    end
end

disp('=================================================================================')
if ~isempty(zvd)
    disp('Final values for zero-variate data points included:')
    for i = 1:length(namesz) % go through contents of the global zvd
        a = eval(['zvd.',namesz{i}]); % extract zero-variate info
        fprintf('%-6s %10.4g (data point: %1.4g, sd: %1.4g) \n',namesz{i},a(3),a(1),a(2))
    end
    disp('=================================================================================')
end

disp(['Time required: ' secs2hms(toc)])
diary off
