% Build-Yer-Own-Model
% 
% General framework for simulating model systems in terms of ordinary
% differential equations (ODEs). This script requires call_deri.m and
% derivatives.m to be specified. Additionally, for fitting, the files
% start_calc.m, packunpack.m, transfer.m and sec2hms.m are used. The
% function calc_proflik.m is optional and can be called to calculate
% profile likelihoods.
%
% Fitting relies on the multinomial likelihood for survival and independent
% normal distributions (if needed after transformation) for continuous
% data. Results are shown on screen but also saved to a log file
% (results.out).
%
% Author: Tjalling Jager (www.debtox.info)
% Date:   Mar 2015

% ========================================================================
%                         THIS EXAMPLE
% ========================================================================
% This script analyses the simulated data sets for the workshop paper. For
% a general package to perform GUTS analyses, please download the latest
% version of BYOM and the GUTS packages from www.debtox.info/byom.html
% ========================================================================

clear all           % clear the memory
global DATA W X0mat % make the data set and initial states global variables
global glo          % allow for global parameters in structure glo
global pri zvd      % global structures for optional priors and zero-variate data
diary off           % turn of the diary function (if it is accidentaly on)
set(0,'DefaultFigureWindowStyle','docked') ; 
% use this option to collect all figure into one window with tab controls

% Set path to the BYOM/engine directory. The function pathdefine
% only works when the mydata file is in a folder below the BYOM folder.
pathdefine

% add dataset folder to the path
addpath('..\datasets','-begin'); 

% ========================================================================
%                       ENTER DATA SET
% ========================================================================

% Data are entered in matrix form, time in rows, scenarios (exposure
% concentrations) in columns. First column are the exposure times, first
% row are the concentrations or scenario numbers.

% Number top left of matrix indicates how to calculate the likelihood:
% -1 ) multinomial likelihood (for survival data)
% 0  ) log-transform the data, then normal likelihood
% 0.5) square-root transform the data, then normal likelihood
% 1  ) no transformation of the data, then normal likelihood

casenm = 'GP_MAL'; % name of the case study (here: Gammarus pulex and malathion)

dataset = 1; % which data set to analyse

% different scenarios for the synthetic data sets
datakey = [1 20 4 8
    2 100 4 8
    3 20 10 8
    4 100 10 8
    5 20 4 5
    6 100 4 5
    7 20 10 5
    8 100 10 5];

N = datakey(dataset,2); % number of test animals
T = datakey(dataset,3); % number of observation times
C = datakey(dataset,4); % number of exposure concentrations (excl. control)

fnm = [casenm,'_',num2str(N),'_',num2str(T),'_',num2str(C)]; % filename for download

for caseno = 1:3 % run through the three replicate sets
    
    % load correct data from text file
    DATA{1} = (load([fnm,'_',char(64+caseno),'.txt']))';
    
    % scaled internal concentrations can have no observations
    DATA{2} = 0;
    
    % Initial states, scenarios in columns, states in rows
    X0mat(1,:) = sort(DATA{1}(1,2:end)); % scenarios (concentrations)
    X0mat(2,:) = 1; % initial survival probability
    X0mat(3,:) = 0; % initial scaled internal concentration
    
    % ========================================================================
    %                   INITIAL VALUES FOR MODEL PARAMETERS
    % ========================================================================
    
    % model parameters are part of a `structure` for easy reference
    % syntax: par.name = [startvalue fit(0/1) minval maxval];
    
    % if parameter K is not specified, package will use the scaled TK model
    % if parameter Ci0 is not specified, package will use value in X0mat as starting value
    % for constant exposure (as in this file), in call_deri, useode can be set to 0 (faster)
    
    % global parameter
    glo.sel   = 3; % select death mechanism: 1) SD 2) IT 3) mixed
    glo.dis   = 2; % NEC distribution for IT: 1) log-logistic 2) log-normal
    % globals used in call_deri
    glo.locDM = 2; % location of the dose metric in the state variable list
    glo.locS  = 1; % location of survival probability in the state variable list
    glo.plt = [1 1 0];
    
    switch glo.sel
        case 1 % SD
            par.ke    = [1  1 0.001 10];    % elimination rate constant, d-1
            par.z     = [0.60    1 0 1e6];   % NEC for survival (ug/L)
            par.hb    = [0.0035 1 0 1e6];   % background hazard rate (1/d)
            par.kk    = [0.39 1 0 1e6];   % killing rate (L/ug/d) (SD only)
            par.Fs    = [1    0 0 100];   % fraction spread of NEC distribution (IT only)
        case 2 % IT
            par.ke    = [1  1 0.001 10];    % elimination rate constant, d-1
            par.z     = [0.60    1 0 1e6];   % NEC for survival (ug/L)
            par.hb    = [0.0035 1 0 1e6];   % background hazard rate (1/d)
            par.kk    = [0 0 0 1e6];   % killing rate (L/ug/d) (SD only)
            par.Fs    = [2    1 1 100];   % fraction spread of NEC distribution (IT only)
        case 3 % mixed
            par.ke    = [1  1 0.001 10];    % elimination rate constant, d-1
            par.z     = [0.60    1 0 1e6];   % NEC for survival (ug/L)
            par.hb    = [0.0035 1 0 1e6];   % background hazard rate (1/d)
            par.kk    = [0.39 1 0 1e6];   % killing rate (L/ug/d) (SD only)
            par.Fs    = [2    1 1 100];   % fraction spread of NEC distribution (IT only)
            
            % slightly different starting values to find optimum for
            % several data sets
            %
            % for analysis 1
            if caseno == 2 && dataset == 1
                par.hb    = [0.00035 1 0 1e6];   % background hazard rate (1/d)
            end
            
            % for analysis 6
            if caseno == 2 && dataset == 6
                par.Fs    = [1.1 1 1 100];   % background hazard rate (1/d)
            end
            
    end
    
    % specify what to calculate and what to plot
    t = linspace(0,max(DATA{1}(:,1)),100); % time vector for the model curves in days
    fit   = 1; % set to 1 to fit the parameters, 0 to simulate only
    sho   = 1; % set to 1 to show all scenarios, 0 to only plot model for scenarios with data
    repls = 1; % set to 1 to plot replicates, 0 to plot mean responses
    
    % specify the y-axis labels for each state variable
    ylab{1} = 'survival probability';
    ylab{2} = 'scaled internal concentration (nM)';
    % specify the x-axis label (same for all states)
    xlab    = 'time (days)';
    leglab1 = 'Conc. '; % legend label before the 'scenario' number
    leglab2 = ' nM'; % legend label after the 'scenario' number
    
    % ========================================================================
    %                CALCULATE AND PLOT
    % ========================================================================
     
    mfilename_base = mfilename; % remember the filename for THIS file for the plots
    % it = 0; % show iterations of the optimisation (1, default) or not (0)
    calc_and_plot; % call the script which calculates and plots
    
    sig_z  = log(par_out.Fs(1))/1.96;  % sd on ln scale, calculated from Fs
    var_z = sig_z^2; % variance on ln scale
    mu_z  = log(par_out.z(1)); % mean threshold on ln scale
    vari  = (exp(var_z)-1)*exp(2*mu_z+var_z);
    sd    = sqrt(vari);
    mn    = exp(mu_z+var_z/2);
    disp(' ')
    disp(['Mean of lognormal distribution: ',num2str(mn)])
    disp(['S.D. of lognormal distribution: ',num2str(sd)])
    disp(' ')
    disp('Predicted survival probability at each data point')
    disp(plotcon_tmp)
    
    % ========================================================================
    %                STANDARD ERRORS AND CONFIDENCE INTERVALS
    % ========================================================================
    % By profiling you make robust confidence intervals for one or more of your
    % parameters. Use the name of the parameter as it occurs in your parameter
    % structure "par" above. You do not need to run the entire script before
    % you can make a profile. However, make sure that you have run at least one
    % optimisation before you make profiles (start_calc sets globals that are
    % needed for this calculation too).
    % Note 1: profiling for complex models is a slow process, so grab a coffee!
    % Note 2: if the profile finds a better solution, it immediately saves it
    % to the lof file profiles_newopt.out. You can break of the analysis by
    % pressing ctrl-c and use the values from the log file to restart.
    
    % run profiles for all fitted parameters.
    if ~exist('par_coll')
        par_coll = [];
    end
    global names % makes name field in the par structure visible here too
    for i = 1:length(names) % run through all parameters
        if eval(['par_out.',names{i},'(2)']) == 1 % if the parameter is fitted ...
            [confD,confU] = calc_proflik(par_out,names{i}); % calculate a profile
            % collect all results in a matrix for later analysis
            par_coll = [par_coll;N T C caseno FVAL i eval(['par_out.',names{i},'(1)']) min(confD) max(confU)];
        end
    end
    
    % ========================================================================
    % The slice samples can be used for a Bayesian analysis: provides a sample
    % from the posterior distribution. A .mat file is saved which contains the
    % sample, to use later for e.g., intervals on model predictions.
    
    % calc_slice(par_out,[500,1,0],mfilename); % second argument is vector with
    %           % number of samples, thinning, and burn-in samples
    % calc_conf % calculate and plot confidence intervals on model curves
    %
    % ========================================================================
    %                CALCULATE LCx
    % ========================================================================
    
    % Tend     = [4 10];  % times at which to calculate LCx, relative to control
    % Feff     = 0.50;    % effect level (>0 en <1), x/100 in LCx
    % LCx_plot = 0;       % set to 1 to make a plot of ECx vs time
    % calc_conf_lc50;     % calculates LCx values, CI requires that there is a mat file with sample
    
end

% write the results to a text file for later interpretation
format short g
fid = fopen([fnm,'_results.txt'],'wt'); % file to write results into
wrtstr = '%g\t';
% in format helpful for R (tab delimited)
for iwrt = 1:size(par_coll,2)-2
    wrtstr = [wrtstr,'%g\t'];
end
wrtstr = [wrtstr,'%g\n'];
fprintf(fid,wrtstr,par_coll');
fclose(fid);